@@ -11,6 +11,8 @@ $mirrors = {
   },
   'Asia' => {
     'China' => {
+      'http://mirror.lzu.edu.cn/CTAN/' => 1,
+      'http://mirrors.opencas.cn/ctan/' => 1,
       'http://mirrors.tuna.tsinghua.edu.cn/CTAN/' => 1,
       'http://mirrors.ustc.edu.cn/CTAN/' => 1
     },
@@ -29,23 +31,17 @@ $mirrors = {
     },
     'Korea' => {
       'http://ftp.kaist.ac.kr/pub/tex-archive/' => 1,
-      'http://ftp.ktug.org/tex-archive/' => 1,
       'http://ftp.neowiz.com/CTAN/' => 1
     },
+    'Malaysia' => {
+      'http://wbc.upm.edu.my/tex-archive/' => 1
+    },
     'Philippines' => {
       'http://mirror.pregi.net/tex-archive/' => 1
     },
     'Taiwan' => {
       'ftp://ftp.ccu.edu.tw/pub/tex/' => 1,
       'http://ftp.yzu.edu.tw/CTAN/' => 1
-    },
-    'Thailand' => {
-      'http://ctan.megagod.net/tex-archive/' => 1
-    }
-  },
-  'Australasia' => {
-    'New Zealand' => {
-      'http://mirror.aut.ac.nz/CTAN/' => 1
     }
   },
   'Europe' => {
@@ -67,9 +63,9 @@ $mirrors = {
     'France' => {
       'http://ctan.mines-albi.fr/' => 1,
       'http://distrib-coffee.ipsl.jussieu.fr/pub/mirrors/ctan/' => 1,
-      'http://ftp.oleane.net/pub/CTAN/' => 1,
       'http://mirror.ibcp.fr/pub/CTAN/' => 1,
-      'http://mirrors.ircam.fr/pub/CTAN/' => 1
+      'http://mirrors.ircam.fr/pub/CTAN/' => 1,
+      'http://mirrors.standaloneinstaller.com/ctan/' => 1
     },
     'Germany' => {
       'ftp://ftp.fu-berlin.de/tex/CTAN/' => 1,
@@ -107,7 +103,8 @@ $mirrors = {
     },
     'Russia' => {
       'http://ctan.altspu.ru/' => 1,
-      'http://mirror.macomnet.net/pub/CTAN/' => 1
+      'http://mirror.macomnet.net/pub/CTAN/' => 1,
+      'http://mirrors.mi.ras.ru/CTAN/' => 1
     },
     'Serbia' => {
       'http://ctan.mirror.ftn.uns.ac.rs/' => 1
@@ -130,8 +127,13 @@ $mirrors = {
     }
   },
   'North America' => {
+    'Australia' => {
+      'http://mirror.aarnet.edu.au/pub/CTAN/' => 1
+    },
     'Canada' => {
+      'http://ctan.forsale.plus/' => 1,
       'http://ctan.math.ca/tex-archive/' => 1,
+      'http://ctan.mirror.colo-serv.net/' => 1,
       'http://ctan.mirror.rafal.ca/' => 1,
       'http://mirror.its.dal.ca/ctan/' => 1,
       'http://muug.ca/mirror/ctan/' => 1
@@ -139,6 +141,9 @@ $mirrors = {
     'Costa Rica' => {
       'http://mirrors.ucr.ac.cr/CTAN/' => 1
     },
+    'New Zealand' => {
+      'http://mirror.aut.ac.nz/CTAN/' => 1
+    },
     'USA' => {
       'http://ctan.mackichan.com/' => 1,
       'http://ctan.math.washington.edu/tex-archive/' => 1,
@@ -148,7 +153,7 @@ $mirrors = {
       'http://mirror.jmu.edu/pub/CTAN/' => 1,
       'http://mirror.unl.edu/ctan/' => 1,
       'http://mirror.utexas.edu/ctan/' => 1,
-      'http://mirrors.acm.jhu.edu/ctan/' => 1,
+      'http://mirrors.concertpass.com/tex-archive/' => 1,
       'http://mirrors.ibiblio.org/pub/mirrors/CTAN/' => 1,
       'http://mirrors.rit.edu/CTAN/' => 1
     }
