@@ -1,8 +1,8 @@
 #!/usr/bin/env perl
-# $Id: install-menu-text.pl 42431 2016-11-03 23:11:36Z karl $
+# $Id: install-menu-text.pl 43566 2017-03-21 17:08:03Z karl $
 # install-menu-txt.pl
 #
-# Copyright 2007-2016 Norbert Preining, Karl Berry
+# Copyright 2007-2017 Norbert Preining, Karl Berry
 # Copyright 2007-2008 Reinhard Kotucha
 # This file is licensed under the GNU General Public License version 2
 # or any later version.
@@ -477,7 +477,7 @@ sub directories_menu
     'Q' => \&quit
       );
 
-  menu_head "Directories setup:";
+  menu_head "Directories customization:";
   if (!TeXLive::TLUtils::texdir_check($vars{'TEXDIR'})) {
     print "!! The default location as given below is forbidden or
 !! can't be written to.
@@ -762,7 +762,7 @@ sub options_menu {
       );
 
   clear_screen;
-  menu_head "Options setup:";
+  menu_head "Options customization:";
 
   print <<"EOF";
  <P> use letter size instead of A4 by default: $b_letter
@@ -995,7 +995,7 @@ sub main_menu {
 ======================> TeX Live installation procedure <=====================
 
 ======>   Letters/digits in <angle brackets> indicate   <=======
-======>   menu items for commands or options            <=======
+======>   menu items for actions or customizations      <=======
 
  Detected platform: $this_platform
  $warn_nobin
@@ -1003,11 +1003,11 @@ EOF
 
   if (!$vars{'in_place'}) {
     print <<"EOF";
- <B> binary platforms: $vars{'n_systems_selected'} out of $vars{'n_systems_available'}
+ <B> set binary platforms: $vars{'n_systems_selected'} out of $vars{'n_systems_available'}
 
- <S> set installation scheme ($vars{'selected_scheme'})
+ <S> set installation scheme: $vars{'selected_scheme'}
 
- <C> customizing installation collections
+ <C> set installation collections:
      $vars{'n_collections_selected'} collections out of $vars{'n_collections_available'}, disk space required: $vars{'total_size'} MB
 EOF
 
@@ -1018,7 +1018,7 @@ EOF
     }
     print <<"EOF";
 
- <D> directories:
+ <D> set directories:
    TEXDIR (the main TeX directory):
 EOF
 
